/****************************************************************************
*    Copyright (c) 1999 CASEMaker Inc.                                      *
*    http://www.dbmaker.com  http://www.dbmaker.com.tw                      *
*    You may distribute under the terms of the GNU General Public License.  *
*                                                                           *
*    mmhtml.c                                                               *
*    function to get pure text of html file                                 *
****************************************************************************/
#include "webut.h"
#include <string.h>

#ifdef WIN32
#define OUT_NEW_LINE() { if (t+2 >= bufsize) break; \
                         buf[t++] = '\r'; buf[t++] = '\n'; }
#else
#define OUT_NEW_LINE() { buf[t++] = '\n'; if (t >= bufsize) break; }
#endif

#define OUT_SPACE() { buf[t++] = ' '; if (t >= bufsize) break; }

/****************************************************************************
 Find the content between the TITLE tag and its end tag
 ****************************************************************************/
void findTitle( 
  char *html,     /* IN html content */
  int  slen,      /* IN html content length */
  char *title)    /* OUT title of html file */
{
  int tlen=0, s;

  for (s = 0 ; s < slen ; s++)
    if (html[s] == '<')                   /* is a tag */
      {
      s++;   
#ifndef WIN32
      if (strncasecmp(html+s, (char *)"title", 5) == 0)   /* is TITLE tag */
#else
      if (strnicmp(html+s, (char *)"title", 5) == 0)      /* is TITLE tag */
#endif
        {
        s += 5;
        while (html[s] != '>')
          s++;
        s++;
        /* copy title data until end of title tag */
        while (s < slen && html[s] != '<' && tlen < LEN_TITLE -1 )
          title[tlen++] = html[s++];
        break;                            /* end of title */
        }
      /* check if comment tag <!-comment->, skip this tag           */
      else if (html[s] == '!' && html[s+1] == '-')  /* is a comment */
        {
        s += 2;
        while (s < slen && (html[s] != '-' || html[s+1] != '>'))
          s++;
        }
      }
  title[tlen] = 0;
}

/***********************************************************************.
 *  html2text  -- convert html source to text and find title of html file
 *
 *  Note: 1. output will include text of html TITLE and BODY
 *        2. output text is null terminated
 *        3. add NEWLINE for BR, /TR
 *           add SPACE for /TD, '\n', control char
.***********************************************************************/
void html2text(
  char *html,              /* IN  input html, needn't NTS */
  int   slen,              /* IN  length of html */
  char *buf,               /* OUT output text buffer */
  int   bufsize,           /* IN  size of output buffer */
  int  *tlen)              /* OUT length of text after convert */
{
#define MAX_TAG 10         /* max length of tag keyword */
#define MAX_ESC 10         /* max length of escape identifier */

  int  s,t;                /* index of source and target */
  int  i;
  char SpecCh[MAX_ESC];    /* speical char with '&' prefix */
  char ch;
  int  fgShow = 0;         /* need to display text */
  int  fgChkTitle = 1;     /* need to check TITLE tag */

  bufsize--;               /* reserve one char for null terminate */

  for (s = 0, t = 0 ; s < slen ; s++)
    {
    /*******************************************************************
     *  process HTML tag
     *******************************************************************/
    if ((ch = html[s]) == '<')           /* start of tag */
      {
      int k = s+1;         /* start of tag keyword */
      int len;             /* length of tag keyword */
      char tag[MAX_TAG];   /* tag keyword */

      /* check comment tag <!-- comment -->   and skip comment   #002  */
      if (s+3 < slen && html[s+1]=='!' && html[s+2]=='-' && html[s+3]=='-')
        {
        s += 4;
        while (s < slen &&
               !(html[s]=='>' && html[s-1]=='-' && html[s-2]=='-'))
          s++;
        continue;          /* skip the comment */
        }

      len = 0;
      /* copy tag keyword */
      while (++s < slen && (isalnum((unsigned)html[s]) || html[s]=='/' ))
        {
        if (len < MAX_TAG-1)
          tag[len++] = toUpper(html[s]);
        }
      tag[len] = 0;
      while (s < slen && html[s] != '>')       /* skip tag argument */
        s++;
      if (len == 2 && memcmp(tag,"BR",2) == 0)
        { OUT_NEW_LINE(); }
      else if (len == 4 && memcmp(tag,"BODY",4) == 0)
        fgShow = 1;
      else if (len == 3 && (memcmp(tag,"/TR",3) == 0))
        { OUT_NEW_LINE(); }
      else if (len == 3 && (memcmp(tag,"/TD",3) == 0 ||
                            memcmp(tag,"/TH",3) == 0                             
			  ))
        { OUT_SPACE(); }
      else if (fgChkTitle)
        {
        if (len == 5 && memcmp(tag,"TITLE",5) == 0)
          fgShow = 1;
        else if (len == 6 && memcmp(tag,"/TITLE",6) == 0)
          {
          OUT_NEW_LINE();
          fgShow = fgChkTitle = 0;
          }
        } /* fgChkTitle */
      }
    /*******************************************************************
     *  process special char with Escape char '&'
     *******************************************************************/
    else if (ch == '&')                  /* is Escape char             */
      {
      i = 0;
      while (++s < slen && html[s] != ';' && i < MAX_ESC-1)
        SpecCh[i++] = toUpper(html[s]); 
      SpecCh[i] = 0;
      if (strcmp(SpecCh,"AMP") == 0)
        buf[t++] = '&';
      else if (strcmp(SpecCh,"NBSP") == 0)
        buf[t++] = ' ';
      else if (strcmp(SpecCh,"LT") == 0)
        buf[t++] = '<';
      else if (strcmp(SpecCh,"QUOT") == 0)
        buf[t++] = '"';
      if (t >= bufsize)
        break;
      }
    /*******************************************************************
     *  process normal char
     *******************************************************************/
    else if (fgShow)                     /* need to output text        */
      {
      if ((unsigned char)ch > '\r')      /* printable character        */
        {
        buf[t++] = ch;
        if (t >= bufsize)
          break;
        }
      else if (t>0 && isalnum((unsigned)(buf[t-1]))) /* newline with alpha in front*/
        { OUT_SPACE(); }
      }
    }
  buf[t] = 0;                            /* null termianted            */
  *tlen = t;
}

