/****************************************************************************
*    Copyright (c) 1999 CASEMaker Inc.                                      *
*    http://www.dbmaker.com  http://www.dbmaker.com.tw                      *
*    You may distribute under the terms of the GNU General Public License.  *
*                                                                           *
*    search.c                                                               *
*    CGI program which are used for web client to search the websearch      *
*    database to find out the files contain search pattern.                 *
*****************************************************************************/
#include "webut.h"
#include "dmwebut.h"

DWitem item[MAX_ITEMS];
int    total_item;
 
/*=================================================================
 The variables below are used for multi-language message support 
 ==================================================================*/
static char NONE[LEN_MES];      /* represent "no data found" */
static char TOOMANY[LEN_MES];   /* represent "found too many data" */
static char PAGE[LEN_MES];      /* represent "page" */
static char ITEMS[LEN_MES];     /* represent "item" */
static char PIPATTERN[LEN_MES]; /* represent "please input pattern word" */
static char SPATTERN[LEN_MES];  /* represent "search pattern" */
static char REQUERY[LEN_MES];   /* represent "re-Query" */
static char ONE[LEN_MES];       /* represent "the first page" */
static char PREV[LEN_MES];      /* represent "the previous page" */
static char NEXT[LEN_MES];      /* represent "the next page" */
static char LAST[LEN_MES];      /* represent "the last page" */
static char SCORE[LEN_MES];     /* represent "score" */
static char tempDir[LEN_FNAME]; /* represent "temp directory" */
static char homeDir[LEN_FNAME]; /* represent "homepage directory" */
static char preTag[LEN_TAG];    /* represent "preTag" in front of searched pattern word */
static char endTag[LEN_TAG];    /* represent "endTag" in back of searched pattern word */

/*****************************************************************************
* static function prototype
******************************************************************************/

static int proClient(char *pattern, int sensitive);
/* used for web client to process search operation */

static void mapClientText(HENV henv, HDBC hdbc);
/* retrieve message by assigned language version 
   and get values of static variable "tempDir", "homeDir", 
   "preTag", and "endTag"   */

static void searchProcess(HENV henv, HDBC hdbc, char *pattern, int *got,
                          int sensitive, int weight, int hitPerP);
static void modifyPattern(char *pattern, char *CGIPattern);
static void removeTmpFile();
static int  NewTmpFile(FNAME fn, FILE **tmpf);
static int  writeItem(int num, char *title, char oidBuf[16],
                      char *fname, int weight, FILE *tf);
static void addLog(HENV henv, HDBC hdbc, char *pattern, int got);

static void oid2string(OID oid, char *buf);
static void showTemplateHeader(HENV henv, HDBC hdbc);
static void showRequery(FNAME TmpFn);
static void readItem(HENV henv, HDBC hdbc, FNAME TmpFn, int pn, int hitPerP);
static void filterName(FNAME relativeName, char *originName);
static void findDescription(HENV henv, HDBC hdbc, char *oid, char *pattern, 
						              	int sensitive);
static void showForm(int pageNumber, int hitPerP, int total, FNAME TmpFn);
static void showTemplateFooter(HENV henv, HDBC hdbc);

/* used for web client to process page-down and page-up search */
static int  getFullPattern(char *pattern, char *fuPattern, FNAME TmpFn);
static void proContinue(int pageNumber, int hitPerP, int total, FNAME TmpFn);

/****************************************************************************
* Main function 
*    Function:
*        . Select content which contain search pattern word from table WebPage
*          and save the search result into a temporary file locating in temp
*          directory 
*        . Find description of content which contain search pattern word 
*        . Show the first page of search result 
* 
*****************************************************************************/
void main()
{
  int   i;                    /* just used for count */
  int   sensitive=0;          /* decide whether case sensitive */
  int   pageNumber=-1;        /* represent page number of result result page */
  int   hitPerP=0;            /* hit count item per page */
  int   total=0;              /* total count of search result */
  int   requery=0;            /* decide whether process re-Query */
  char  pattern[LEN_PAT];     /* pattern word user want to search */
  char  fuPattern[LEN_FUPAT]; /* full pattern word user want to search */
  FNAME TmpFn;                /* temporary file name */

  TmpFn[0]=0;    /* used to check whether this is a re-Query */

  printf("Content-type: text/html\n\n");
  DWGetItem();   /* get CGI string name and value */

  for (i=0; i<total_item; i++)
    {
    if (strcmp(item[i].name, "pattern")==0)
      strcpy(pattern, item[i].value);
    else if (strcmp(item[i].name, "c-sensitive")==0)
      sensitive=atoi(item[i].value);

    /* TmpFn and addPattern is used to check whether the query is a requery */
    else if (strcmp(item[i].name, "TmpFn")==0)
      strcpy(TmpFn, item[i].value);
    else if (strcmp(item[i].name, "addPattern")==0)
      {
      strcpy(pattern, item[i].value);
      requery=1;
      }

    /* pageNumber, hitPerP, and total is used for page-down and page-up */
    /* pageNumber != -1 means the operation is a page-down or page-up   */
    else if (strcmp(item[i].name, "pageNumber")==0) 
      pageNumber=atoi(item[i].value); 
    else if (strcmp(item[i].name, "hitPerP")==0) 
      hitPerP=atoi(item[i].value); 
    else if (strcmp(item[i].name, "total")==0) 
      total=atoi(item[i].value); 
    }

  /* TmpFn[0] used the check the query is a requery */
  if ((TmpFn[0]) && requery)
    {
    getFullPattern(pattern, fuPattern, TmpFn);
    if (getFullPattern(pattern, fuPattern, TmpFn))
      goto EXIT;  
    } 
  else 
    {
    strcpy(fuPattern, pattern);
    fuPattern[strlen(pattern)]=0;
    }

  if (pageNumber != -1)  
    proContinue(pageNumber, hitPerP, total, TmpFn); 
  else
    proClient(fuPattern, sensitive); 

EXIT:
  return;
}

/****************************************************************************
*  Process All jobs for Web Client, describe below:
*       # Connect to Database WebSearch
*       # insert values into table Log(opTime, pattern, got)
*       # select values from table Output(header, footer)
*       # select values from table Param(name, chValue, intValue)
*       # select data from table WebPage
*  Show the first page of search result
*  Output how many items in searched result
*****************************************************************************/
static int proClient(
  char *pat,      /*  IN pattern word user want to search */
  int sensitive)  /*  IN whether the search is case-sensitive */
{
  char   buf[1000];
  char   name[LEN_NAME];
  int    got=0;       /* how many item got from search result */
  int    intValue;  
  int    hitPerP=10;  /* hit count per page */
  int    weight=10;   /* weight of title */
  int    log=0;       /* whether log operation of web client */
  HENV   henv;        
  HDBC   hdbc;
  HSTMT  hstmt;
  SDWORD cbname;
  SDWORD cbintValue;
  SDWORD rc=0;
  
  if (SQLAllocEnv(&henv))
    printf("allocate envrionment handle fail !! \n");
  if (SQLAllocConnect(henv,&hdbc))
    printf("allocate connection handle fail !! \n");

  /*=== User Guest with priviledge DBA Connect to Database DBNAME ===*/
  SQLConnect(hdbc, (UCHAR *)DBNAME, SQL_NTS, (UCHAR *)"guest", 
             SQL_NTS, (UCHAR *)"", SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, NULL);  
  SQLSetConnectOption(hdbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_OFF);
  checkrc(RTN_OK, henv, hdbc, NULL);  
  SQLAllocStmt(hdbc,&hstmt);
  checkrc(RTN_OK, henv, hdbc, hstmt);  

  mapClientText(henv, hdbc);  
  showTemplateHeader(henv, hdbc); 

  sprintf(buf, "select name, intValue from SYSADM.Params");
  SQLExecDirect(hstmt, (UCHAR *)buf, SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, hstmt);  

  SQLBindCol(hstmt, 1, SQL_C_CHAR, name, LEN_NAME, &cbname);
  checkrc(RTN_OK, henv, hdbc, hstmt);  
  SQLBindCol(hstmt, 2, SQL_C_LONG, &intValue, 0, &cbintValue);
  checkrc(RTN_OK, henv, hdbc, hstmt);  
    
  while (TRUE)
    {
    rc=SQLFetch(hstmt);
    if (rc==SQL_SUCCESS || rc==SQL_SUCCESS_WITH_INFO)
      {
      if (strcmp(name, "weight")==0) 
        weight=intValue;
      else if (strcmp(name, "hitPerP")==0) 
        hitPerP=intValue;
      else if (strcmp(name, "log") ==0)
        log=intValue;
      }
    else 
      {
      if (rc==SQL_NO_DATA_FOUND)
        break;      
      else 
        checkrc(RTN_OK, henv, hdbc, hstmt); 
      }
    }
  SQLFreeStmt(hstmt, SQL_DROP);
  checkrc(RTN_OK, henv, hdbc, hstmt);   

  /*================================================================
    Search table WebPage to find files which contain words 
    that match pattern 
    ================================================================*/
  searchProcess(henv, hdbc, pat, &got, sensitive, weight, hitPerP);
  
  showTemplateFooter(henv, hdbc);  

  /*=== Add entry in table Log ===*/
  if (log) 
    addLog(henv, hdbc, pat, got);

EXIT:
  SQLDisconnect(hdbc);
  SQLFreeConnect(hdbc);
  SQLFreeEnv(henv);
  return got;
}

/****************************************************************************
 Map message according language version which administrator assigned 
 ****************************************************************************/
static void mapClientText(HENV henv, HDBC hdbc)
{
  int    len;                  /* use to count character count of string */
  char   version[LEN_PARAM];   /* language version */
  char   name[LEN_NAME];       /* name select from table mapText */
  char   display[LEN_MES];     /* showing message */
  char   path[LEN_FNAME];      /* retrieve path from table Path */
  char   chValue[LEN_CHVALUE];
  char   buf1[1000];  
  SDWORD cbversion;
  SDWORD cbdisplay;
  SDWORD cbname;
  SDWORD cbpath;
  SDWORD cbchValue;
  SDWORD rc=0;
  HSTMT  hstmt;

  SQLAllocStmt(hdbc,&hstmt);
  checkrc(RTN_OK, henv, hdbc, hstmt);  

  sprintf(buf1, "select chValue from SYSADM.Params where name='version'");
  SQLExecDirect(hstmt, (UCHAR *)buf1, SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  
  SQLBindCol(hstmt, 1, SQL_C_CHAR, version, 8, &cbversion);
  checkrc(RTN_OK, henv, hdbc, hstmt);   

  SQLFetch(hstmt);
  checkrc(RTN_OK, henv, hdbc, hstmt); 

  SQLFreeStmt(hstmt, SQL_UNBIND);
  checkrc(RTN_OK, henv, hdbc, hstmt);   
  SQLFreeStmt(hstmt, SQL_CLOSE);
  checkrc(RTN_OK, henv, hdbc, hstmt);   
  
  if (version[0]=='c')  /* version =="chinese" */
    sprintf(buf1, "select name, display from SYSADM.mapText where language='chinese'");
  else if (version[0]=='e') /* version == "english" */
    sprintf(buf1, "select name, display from SYSADM.mapText where language='english'");

  SQLExecDirect(hstmt, (UCHAR *)buf1, SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, hstmt);   
  
  SQLBindCol(hstmt, 1, SQL_C_CHAR, name, LEN_NAME, &cbname);
  checkrc(RTN_OK, henv, hdbc, hstmt);   
  SQLBindCol(hstmt, 2, SQL_C_CHAR, display, LEN_MES, &cbdisplay);
  checkrc(RTN_OK, henv, hdbc, hstmt);   
    
  while (TRUE)
    {
    rc=SQLFetch(hstmt);
    if (rc==SQL_SUCCESS || rc==SQL_SUCCESS_WITH_INFO)
      {
      if (strcmp(name, "NONE") ==0)
        strcpy(NONE, display); 
      else if (strcmp(name, "TOOMANY") ==0)
        strcpy(TOOMANY, display);
      else if (strcmp(name, "ITEMS") ==0)
        strcpy(ITEMS, display);
      else if (strcmp(name, "PIPATTERN") ==0)
        strcpy(PIPATTERN, display); 
      else if (strcmp(name, "PATTERN") ==0)
        strcpy(SPATTERN, display); 
      else if (strcmp(name, "REQUERY") ==0)
        strcpy(REQUERY, display); 
      else if (strcmp(name, "ONE") ==0)
        strcpy(ONE, display); 
      else if (strcmp(name, "PREV") ==0)
        strcpy(PREV, display);
      else if (strcmp(name, "NEXT") ==0)
        strcpy(NEXT, display);
      else if (strcmp(name, "LAST") ==0)
        strcpy(LAST, display);
      else if (strcmp(name, "PAGE") ==0)
        strcpy(PAGE, display);
      else if (strcmp(name, "SCORE") ==0)
        strcpy(SCORE, display);
      }
    else 
      {
      if (rc==SQL_NO_DATA_FOUND)
        break;      
      else 
        checkrc(RTN_OK, henv, hdbc, hstmt); 
      }
    }
  SQLFreeStmt(hstmt, SQL_UNBIND);
  checkrc(RTN_OK, henv, hdbc, hstmt);   
  SQLFreeStmt(hstmt, SQL_CLOSE);
  checkrc(RTN_OK, henv, hdbc, hstmt);   

  sprintf(buf1, "select type, path from SYSADM.Path");
  SQLExecDirect(hstmt, (UCHAR *)buf1, SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, hstmt);   

  SQLBindCol(hstmt, 1, SQL_C_CHAR, name, LEN_NAME, &cbname);
  checkrc(RTN_OK, henv, hdbc, hstmt);   
  SQLBindCol(hstmt, 2, SQL_C_CHAR, path, LEN_FNAME, &cbpath);
  checkrc(RTN_OK, henv, hdbc, hstmt);   
    
  while (TRUE)
    {
    rc=SQLFetch(hstmt);
    if (rc==SQL_SUCCESS || rc==SQL_SUCCESS_WITH_INFO)
      {
      if (strcmp(name, "tempDir") ==0)
        strcpy(tempDir, path); 
      else if (strcmp(name, "homeDir") ==0)
        strcpy(homeDir, path); 
      }
    else 
      {
      if (rc==SQL_NO_DATA_FOUND)
        break;      
      else 
        checkrc(RTN_OK, henv, hdbc, hstmt);
      }
    }
  SQLFreeStmt(hstmt, SQL_UNBIND);
  checkrc(RTN_OK, henv, hdbc, hstmt);   
  SQLFreeStmt(hstmt, SQL_CLOSE);
  checkrc(RTN_OK, henv, hdbc, hstmt);

  len=strlen(tempDir);
  if ((! isDbcsHi(tempDir[len-2]) &&
      (tempDir[len-1] == SLASH)))
    tempDir[len-1]=0;   

  len = strlen(homeDir);
  if ((! isDbcsHi(homeDir[len-2]) &&
      (homeDir[len-1] == SLASH)))
    homeDir[len-1]=0;   

  sprintf(buf1, "select name, chValue from SYSADM.Params where name='preTag' "
	           "or name='endTag'");
  SQLExecDirect(hstmt, (UCHAR *)buf1, SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, hstmt);  

  SQLBindCol(hstmt, 1, SQL_C_CHAR, name, LEN_NAME, &cbname);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  SQLBindCol(hstmt, 2, SQL_C_CHAR, chValue, LEN_FNAME, &cbchValue);
  checkrc(RTN_OK, henv, hdbc, hstmt);
    
  while (TRUE)
    {
    rc=SQLFetch(hstmt);
    if (rc==SQL_SUCCESS || rc==SQL_SUCCESS_WITH_INFO)
      {
      if (strcmp(name, "preTag")==0)
        strcpy(preTag, chValue); 
      else if (strcmp(name, "endTag")==0) 
        strcpy(endTag, chValue); 
      }
    else 
      {
      if (rc==SQL_NO_DATA_FOUND)
        break;      
      else checkrc(RTN_OK, henv, hdbc, hstmt); 
      }
    }
  
  SQLFreeStmt(hstmt, SQL_DROP);
  checkrc(RTN_OK, henv, hdbc, hstmt);   

EXIT:
  return;
}

/************************************************************************ 
 Search table WebPage to find files which contain words that match pattern 
 ************************************************************************/
static void searchProcess(
  HENV henv, 
  HDBC hdbc, 
  char *pattern,    /* search pattern word */
  int  *got,        /* how many count of search result */
  int  sensitive,   /* whether case-sensitive */
  int  weight,      /* weight of title */
  int  hitPerP)     /* hit count per page */
{
  HSTMT  hstmt;
  OID    oidRec;
  SDWORD rc=0;
  SDWORD cbfname;
  SDWORD cbtitle;
  SDWORD cboidRec;
  FILE   *tf=NULL;    /* point to temporary file */
  HEADER header;      /* file header of temporary file */
  FNAME  TmpFn;       /* filename of temporary file */
  int    no;          /* serial number of inserted data */
  int    score;       /* score of searched item */
  int    total=0;     /* total searched item count */
  int    i;               
  char   str1[1000];
  char   fname[LEN_FNAME];  /* filename of searched item */
  char   title[LEN_TITLE];       /* title of searched item */
  char   oidString[17];     
  char   CGIpattern[LEN_FUPAT];  /* convert pattern word which contein '&' to  '%26' */
 
  SQLAllocStmt(hdbc,&hstmt);
  checkrc(RTN_OK, henv, hdbc, hstmt);  

  if (sensitive==1)
    sprintf(str1,"select count(*) from SYSADM.WebPage where content case match '%s'",pattern);
  else
    sprintf(str1,"select count(*) from SYSADM.WebPage where content match '%s'",pattern);
    
  SQLExecDirect(hstmt, (UCHAR *)str1, SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, hstmt);   
  SQLBindCol(hstmt, 1, SQL_C_LONG,  &total, 0, NULL);
  checkrc(RTN_OK, henv, hdbc, hstmt);   
    
  SQLFetch(hstmt);
  checkrc(RTN_OK, henv, hdbc, hstmt); 

  SQLFreeStmt(hstmt, SQL_UNBIND);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  SQLFreeStmt(hstmt, SQL_CLOSE);
  checkrc(RTN_OK, henv, hdbc, hstmt);

  if (total > MAXROW)
    {
    printf("&nbsp;&nbsp;&nbsp;%s<br>", TOOMANY);          
    goto EXIT;
    }
  else if (total == 0)
    {
    printf("%s \n", NONE); 
    goto EXIT;
    }

  /*==========================================================
    Seed the random-number generator with current time so that
    the numbers will be different every time we run.  
    ==========================================================*/
  srand((unsigned)time(NULL));
  i=rand() %137;
  
  /*========================================================== 
    One client user of ten client users have opportunity to 
    remove temp file, 14/137 is almost 1/10.
    ==========================================================*/
  if (i < 15)   
    removeTmpFile();

  if (NewTmpFile(TmpFn,&tf))
    {
    printf("create temp file error, error=%d \n", errno);
    goto EXIT;
    }
    
  header.total=total;
  strcpy(header.pattern,pattern);
  header.hitPerP=hitPerP;
  modifyPattern(pattern, CGIpattern);
  strcpy(header.CGIpattern, CGIpattern);
  header.sensitive=sensitive;
  
  fwrite(&header, sizeof(header), 1, tf); 
  if (sensitive==1)
    {
    sprintf(str1,
    "select num, filename(fobj), title, OID, hitcount(title, '%s', %d)*%d+hitcount(content,'%s', %d) "
    " from SYSADM.WebPage where content case match '%s' order by 5 DESC",
    pattern, sensitive, weight, pattern, sensitive, pattern);
    }
  else 
    {
    sprintf(str1,
    "select num, filename(fobj), title, OID, hitcount(title, '%s', %d)*%d+hitcount(content,'%s', %d) "
    " from SYSADM.WebPage where content match '%s' order by 5 DESC",
    pattern, sensitive, weight, pattern, sensitive, pattern);
    }
  
  SQLExecDirect(hstmt, (UCHAR *)str1, SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, hstmt);

  SQLBindCol(hstmt, 1, SQL_C_LONG,  &no,    0, NULL);    
  checkrc(RTN_OK, henv, hdbc, hstmt);
  SQLBindCol(hstmt, 2, SQL_C_CHAR,  fname, LEN_FNAME, &cbfname);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  SQLBindCol(hstmt, 3, SQL_C_CHAR,  title, LEN_TITLE, &cbtitle);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  SQLBindCol(hstmt, 4, SQL_C_BINARY,&oidRec,8, &cboidRec);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  SQLBindCol(hstmt, 5, SQL_C_LONG,  &score, 0, NULL);
  checkrc(RTN_OK, henv, hdbc, hstmt);

  while (TRUE)
    {
    rc = SQLFetch(hstmt);
    if (rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO)
      {
      (*got)++;
      oid2string(oidRec, oidString);
    
      if (score >=20) score=100;
      else score=score*5;

      if (writeItem(*got, title, oidString, fname, score, tf))
        goto EXIT;
      }
    else 
      {
      if (rc== SQL_NO_DATA_FOUND)  
        break;
      else 
        checkrc(RTN_OK, henv, hdbc, hstmt); 
      }
    }
  SQLFreeStmt(hstmt, SQL_DROP);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  
  if (tf)
    fclose (tf);

  /* show first page of search result */
  showRequery(TmpFn);
  readItem(henv, hdbc, TmpFn, 0, hitPerP);
  showForm(0, hitPerP, total, TmpFn);

EXIT:
  return;
}

/****************************************************************************
 Modify '&' in pattern to become "%26"  
 ****************************************************************************/
static void modifyPattern(
  char *pattern,       /* IN source pattern */
  char *newPattern)   /* OUT modified pattern */
{
  char *token;
  char oldPattern[LEN_FUPAT]; /* original pattern */
 
  newPattern[0]=0;
  strcpy(oldPattern, pattern);
  token=strtok(oldPattern, "&");
  strcpy(newPattern, token);

  while (token!= NULL)
    {
    token=strtok(NULL, "&"); 
    if (token == NULL) 
      break;    
    strcat(newPattern, "%26");
    strcat(newPattern, token);
    }

  if (newPattern == NULL)  /* source pattern doesn't conatin '&' */
    strcpy(newPattern, pattern);
}

/**************************************************************************** 
 Put OIDitem Data to temp file, including num, title, CGIname, score 
 ****************************************************************************/
static int writeItem(
  int  num,         /* IN serial number */
  char *title,      /* IN pointer to title string */
  char *oidString,  /* IN oid string */
  char *fname,      /* IN temporary file name */
  int  score,       /* IN score of searched item */
  FILE *tf)         /* OUT temporary file pointer */
{
  ITEM  item;       /* item data to insert into temporary file */
  int   nWrite;     /* count written to temporary file */
 
  if (tf == NULL)
     {
     printf("<br>file handle is NULL\n<br>");
     return(1);
     } 
  item.num=num;
  strcpy(item.title, title);
  strcpy(item.fname, fname);
  strcpy(item.oid, oidString);
  item.score=score;
  
  nWrite= fwrite(&item, sizeof(item), 1, tf);
  if (nWrite != 1) 
    {
    printf("<br><br><br><h2><font color=\"#ff0000\">Disk Full</font></h2>");
    return(1);
    }
  return(0);
  /*=== Go to the first position of the template file ===*/
}

/****************************************************************************
 Open the temporary file to read associated propery and show item of result 
 with description 
 ****************************************************************************/
static void readItem(
  HENV  henv, 
  HDBC  hdbc, 
  FNAME TmpFn,        /* temporary file name */
  int   pn,           /* page number */
  int   hitPerP)      /* hit count per page */
{
  ITEM   item;
  FILE   *tf=NULL;    
  HEADER header;
  int    i;
  int    showCount;     
  int    total;         /* total searched item count */
  FNAME  relativeName;  /* relative filename according to homepage directory */
  char   href[300];     /* href string link to searched html page */
  
  if ((tf=fopen(TmpFn, "rb"))== NULL)
    {
    printf("File open error, %d\n", errno); 
    goto EXIT;
    }
  
  fseek(tf, 0L, SEEK_SET);
  fread(&header, sizeof(header), 1, tf);

  printf("<h3>%s &nbsp; <font color=\"#ff0000\">%s</font>"
         "</h3>\n", SPATTERN, header.pattern); 
  total=header.total;

  if ((total-(pn+1)*hitPerP) > 0)
    showCount=hitPerP; 
  else 
    showCount= total-pn*hitPerP;
  
  printf("%s <b>%d</b>: [%s <b>%d - %d</b>] / %d<br>\n", 
          PAGE, pn+1, ITEMS, 1+hitPerP*pn, hitPerP*pn+showCount, total);
            
    
  fseek(tf, sizeof(header)+pn*sizeof(item)*hitPerP, SEEK_SET);
  for (i=0; i<showCount; i++)
    {
    fread(&item, sizeof(item), 1, tf); 


	/* filter full path filename to relative file name */
  	filterName(relativeName, item.fname);            

  	sprintf(href, "<a href=\"%s\"> %s </a>", relativeName, item.title); 

    printf("<br><font size=\"4\"><b>%d</b>. <b> %s </b></font>\n", item.num, href); 
    printf("&nbsp;&nbsp;&nbsp;&nbsp; %s <font color=\"#ff0000\">%d</font><br>\n", 
            SCORE, item.score);
    findDescription(henv, hdbc, item.oid, header.pattern, header.sensitive);
    }           
EXIT:    
  if (tf) 
    fclose(tf);         
}

/****************************************************************************
 Filter the full path filename to become relative path filename 
 ****************************************************************************/
static void filterName(
  FNAME relativeName,     
  char *originName)
{
  FNAME tmpName;
  int   i;
  int   len;
   
  len=strlen(originName)-strlen(homeDir);

  strncpy(tmpName, originName+strlen(homeDir), len);
  tmpName[len]=0;

  sprintf(relativeName, "%s", tmpName);
  relativeName[len]=0;

#ifdef WIN32
  for (i=0; i<len; i++)
  {
    if (isDbcsHi(relativeName[i]))
		i++;
    else if (relativeName[i] == SLASH)
		relativeName[i]= '/';
  }
#endif
}

/****************************************************************************
 Remove temporary file to avoid producing large garbage 
 ****************************************************************************/
static void removeTmpFile()
{
#ifdef WIN32  
  HANDLE            dirp=0;
  WIN32_FIND_DATA   dirent;
  LPWIN32_FIND_DATA direntp=&dirent;
  struct stat      statbuf; 
#else
  DIR    *dirp = NULL;
  struct dirent *direntp;
  struct stat statbuf;
#endif
  SDWORD rc=0;
  time_t now;
  time_t created;
  double timeOffset;
  char   fullname[LEN_FNAME];
  char   tempDirectory[LEN_FNAME]; /* temp file directory */
  char   command[LEN_FNAME];
  char   path[LEN_FNAME];
  char   *pDest=NULL;
  char   *fname=NULL;

  strcpy(tempDirectory, tempDir);  
  sprintf(path, "%s%c", tempDirectory, SLASH);

#ifdef WIN32
  strcat(tempDirectory, "\\*");
#endif   

	WS_OPENDIR(tempDirectory, direntp, dirp, rc);
  if (rc)
    { 
    printf("Open Directory %s error<br>\n", tempDirectory); 
    goto EXIT;
    }

  WS_READDIR(dirp, direntp, rc);
  while(rc==0)
    {
    WS_GETFILENAME(direntp, fname);
    if(strcmp(fname,".") && strcmp(fname,".."))
      {
      sprintf(fullname, "%s%s", path, fname);
      if (stat(fullname,&statbuf))
        {
        printf("File %s get stat information error<br>\n", fullname);      
        goto NEXT;
        }
      else   /* successfully get the information of the file */                            
        {
        if ((S_IFDIR & statbuf.st_mode)==0)
          {
          /*=== file is not a directory ===*/
          if ((pDest=strrchr(fname,'.'))  && (pDest+1) )
            {
            if (strncmp(pDest+1, "TMP", 3) ==0)             
              {
              time(&now);
              created=statbuf.st_ctime;
              timeOffset=difftime(now, statbuf.st_ctime);
              if (timeOffset >= TIME_EXPIRE)
                WS_DELETEFILE(fullname, command); 
              }            
            }
          }  
        }
      }     
NEXT:    
    WS_READDIR(dirp, direntp, rc);
    }

EXIT:  
  if (dirp)    
    WS_CLOSEDIR(dirp);
}

/**************************************************************************** 
  Find a description containing search pattern 
 ****************************************************************************/
static void findDescription(
  HENV henv, 
  HDBC hdbc, 
  char *oid, 
  char *pattern, 
  int sensitive)
{
  HSTMT  hstmt;
  SDWORD rc=0;
  SDWORD cbfcontent;
  int    boffset;   /* begin offset of description */
  int    eoffset;   /* end offset of description */
  int    len;
  int    aux;
  int    have;
  int    szFront;
  char   buf[1000];
  char   *fcontent=NULL;
  char   *token=NULL;
  char   *des1=NULL;
  char   *des2=NULL;
  char   *front=NULL;
  char   *back=NULL;
  char   *showPattern=NULL;   /* showPattern is different from pattern when we search multiple patter word */
                              /* For example, pattern is 'a&b', showPattern will be 'a' or 'b' */ 
  char   *cPunctuation[4]={"C","I","H","F"};  /* Chinese punctuation */
  char   punctuation[5]={'.','!','?',';','\n'};    /* English punctuation */

  SQLAllocStmt(hdbc,&hstmt);
  checkrc(RTN_OK, henv, hdbc, hstmt);  

  sprintf(buf,"select hitpos(content,'%s', %d, 1, 0), hitpos(content, '%s', %d, 1, 1) ,bloblen(content) "
              "from SYSADM.WebPage where OID='%.16s'x", pattern, sensitive, pattern, sensitive, oid); 
  SQLExecDirect(hstmt, (UCHAR *)buf, SQL_NTS);               
  checkrc(RTN_OK, henv, hdbc, hstmt);  

  SQLBindCol(hstmt, 1, SQL_C_LONG, &boffset, 0, NULL);
  checkrc(RTN_OK, henv, hdbc, hstmt);  
  SQLBindCol(hstmt, 2, SQL_C_LONG, &eoffset, 0, NULL);
  checkrc(RTN_OK, henv, hdbc, hstmt);  
  SQLBindCol(hstmt, 3, SQL_C_LONG, &len,     0, NULL);
  checkrc(RTN_OK, henv, hdbc, hstmt);  
  
  SQLFetch(hstmt);     
  checkrc(RTN_OK, henv, hdbc, hstmt);

  SQLFreeStmt(hstmt, SQL_UNBIND);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  SQLFreeStmt(hstmt, SQL_CLOSE);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  
  if ((fcontent=malloc(len+1))==NULL)
    printf("fcontent allocatin error");
  
  sprintf(buf, "select content from SYSADM.WebPage where OID='%.16s'x", oid);
  SQLExecDirect(hstmt, (UCHAR *)buf, SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, hstmt);

  SQLBindCol(hstmt, 1, SQL_C_CHAR, fcontent, len+1, &cbfcontent);
  checkrc(RTN_OK, henv, hdbc, hstmt);  

  SQLFetch(hstmt);
  checkrc(RTN_OK, henv, hdbc, hstmt); 
 
  SQLFreeStmt(hstmt,SQL_DROP);
  checkrc(RTN_OK, henv, hdbc, hstmt);

  if ((showPattern=malloc(eoffset-boffset+2))==NULL)
    {
    printf("allocatin of showPattern error");
    goto EXIT;
    }
  strncpy(showPattern, fcontent+boffset-1, eoffset-boffset+1);
  showPattern[eoffset-boffset+1]=0;
  
  /*=========================================================================
   Take the pattern position as target, look forward LEN_FRONT(280) bytes of character, 
   and look backward LEN_BACK(400) bytes to find a description containing the pattern 
   ==========================================================================*/
  szFront= (boffset > LEN_FRONT) ? LEN_FRONT : boffset ;
  front=fcontent+boffset-szFront;
  front[szFront-1]='\0';
    
  back=fcontent+eoffset;
  if (strlen(back) > LEN_BACK )
    back[LEN_BACK]='\0';
  
  for (aux=0; aux<4; aux++)
    {
    have=0;
    token=front;  
    /* have is used to decide the content have the target punctuation*/
    while (strstr(token, cPunctuation[aux]))
      {
      have=1;
      des1=strstr(token, cPunctuation[aux]);
      token=des1+2;  /* filter the punctuation */
      }

    /* des1 is the beginning of description */
    if (have)
      front=des1+2;  /* filter the punctuation */
 
    if (des2=strstr(back, cPunctuation[aux]))  /* des2 is the end of description */
      back[des2-back+2]='\0';
    }

  for (aux=1; aux<5; aux++)
    {
    token=front;  
    if (des1=strrchr(token, punctuation[aux])) /* des1 is the beginning of description */
      front=des1+1;  /* filter the punctuation */
    
    if (des2=strchr(back, punctuation[aux]))  /* des2 is the end of description */
      back[des2-back+1]='\0';  /* filter the content after the punctuation */
    }

  /* code below is used to process '.', Punctuation's priority is lower because it may be in number (e.g. 1.2) */
  if (des1==NULL)  
    {
    token=front;  
    if (des1=strrchr(token, punctuation[0]))    
    /* des1 contains the punctuation, and front is the beginning of description */
      front=des1+1;   /* filter the punctuation */
    }

  if (des2==NULL)
    {
    if (des2=strchr(back, punctuation[0])) 
      back[des2-back+1]='\0';   /* filter the content after the punctuation */
    }
    printf("\n<font size=\"2\">  %s  %s  %s  %s  %s  </font><br>", 
           front, preTag, showPattern, endTag, back);
EXIT:  
  if (showPattern)
    free(showPattern);
  if (fcontent) 
    free(fcontent);
}

/****************************************************************************
 Add entry in table Log(opTime, pattern, got) 
 ****************************************************************************/
static void addLog(
  HENV henv, 
  HDBC hdbc, 
  char *pat,        /* IN  search pattern word */
  int got)          /* OUT got count of searched item */
{
  HSTMT  hstmt;
  SDWORD cbopTime=SQL_NTS;
  SDWORD cbpattern=SQL_NTS;
  struct tm *t;
  long   ltime;  
  char   opTime[LEN_TIME];  /* operation time of web client */
  char   pattern[LEN_FUPAT];/* search pattern word */
  char   str1[1000];
 
  SQLAllocStmt(hdbc,&hstmt);
  checkrc(RTN_OK, henv, hdbc, hstmt);  
  
  strcpy(str1, 
         "insert into SYSADM.Log(opTime, pattern, got) values (?,?,?)");
  SQLPrepare(hstmt, (UCHAR *)str1, SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, hstmt); 
  
  /* =============================================================
      parameter $1, $2, $3 
          $1 is operate time, 
          $2 is pattern which to search 
          $3 is got count
     ============================================================= */
 
  SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_TIMESTAMP,
                   LEN_TIME,  0,  opTime, 0, &cbopTime);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_VARCHAR,
                   LEN_FUPAT, 0, pattern, 0, &cbpattern);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  SQLBindParameter(hstmt, 3, SQL_PARAM_INPUT, SQL_C_LONG, SQL_INTEGER,
                    0,  0,    &got, 0, NULL);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  
  time(&ltime);
  t= localtime(&ltime);
  
  sprintf(opTime, "%.04d-%.02d-%.02d %.02d:%.02d:%.02d",
          1900+t->tm_year, t->tm_mon+1, t->tm_mday,
          t->tm_hour, t->tm_min, t->tm_sec);
  strcpy(pattern, pat);

  SQLExecute(hstmt);
  checkrc(RTN_OK, henv, hdbc, hstmt);

  /*=== Produce a commit operation ===*/
  SQLTransact(henv, hdbc, SQL_COMMIT);
  checkrc(RTN_OK, henv, hdbc, hstmt);
EXIT:
  SQLFreeStmt(hstmt,SQL_DROP);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  return;
}

/***********************************************************************.
 *  NewTmpFile -- create a temp file
 *
 *    temp file name is HSSSSSSS.TMP
 *         where H denotes hours start by symbol 'A'
 *               S denotes HashFunc1(time) % 10000000
 *    return 0 if success, -1 if error
.***********************************************************************/
static int NewTmpFile(
  FNAME   fn,        /* OUT file name of temp file */
  FILE  **tmpf)      /* OUT temp file handle */ 
{
  struct   timeval   t1;
  
#ifdef WIN32
  DW  tz1;
#else
  struct timezone   tz1;
#endif
  struct   tm *t;
  unsigned int  s;
  int      ctr = 0;
  int      len;
  FNAME    fullfn;

  gettimeofday(&t1,&tz1);
  t = localtime((time_t*)&t1.tv_sec);
  s = (t->tm_min*60+t->tm_sec)*2770+(unsigned)(t1.tv_usec )%2770;
  sprintf(fn,"%c%07d",'A'+t->tm_hour, s % 10000000L);

  len=strlen(tempDir);

  if ((! isDbcsHi(tempDir[len-2]) &&
      (tempDir[len-1] != SLASH)))
    tempDir[len]=SLASH; 

  sprintf(fullfn,"%s%s%s",tempDir,fn,".TMP");
  strcpy(fn, fullfn);

  *tmpf = fopen(fullfn,"wb");
  if (*tmpf == NULL)
    {
    printf("open temp file [%s] error, errno==%d", fullfn, errno);
    return(-1);
    }

#ifdef WIN32
  _chmod(fullfn, _S_IREAD | _S_IWRITE);
#else  
  chmod(fullfn, S_IRWXU | S_IRWXG | S_IRWXO);
#endif

  return(0);
}

/****************************************************************************
 print out OID to be a string 
 ****************************************************************************/
static void oid2string(
  OID  oid,
  char *buf)
{
  char *p = (char *)&oid;
  int  i;

  for (i = 0 ; i < 8 ; i++)
     {
     sprintf(buf+i*2,"%02x",(unsigned char)p[i]);
     }
  buf[16]='\0';
} /* oid2str */  

/****************************************************************************
 Show the header content before SEARCHTAG in template html file, and then 
 put the search result after header.
 ****************************************************************************/
static void showTemplateHeader(
  HENV henv, 
  HDBC hdbc)
{
  HSTMT   hstmt;
  SDWORD  cbheader;
  SDWORD  rc =0;
  char    header[1024];
  char    buf[1000];
  
  SQLAllocStmt(hdbc,&hstmt);
  checkrc(RTN_OK, henv, hdbc, hstmt);  

  sprintf(buf, "select header from SYSADM.Output");
  SQLExecDirect(hstmt, (UCHAR *)buf, SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  while (TRUE)
    {
    rc = SQLFetch(hstmt);

    if (rc==SQL_SUCCESS || rc==SQL_SUCCESS_WITH_INFO)
      {
      while (TRUE)
        {  
        rc = SQLGetData(hstmt, 1, SQL_C_CHAR, header, 1024, &cbheader);
        
        if (rc==SQL_SUCCESS)
          {
          printf("%s",header);
          break;
          }
        else if( rc==SQL_SUCCESS_WITH_INFO)
          {
          printf("%s",header);
          }
        else checkrc(RTN_OK, henv, hdbc, hstmt); 
        }  
      }
    else 
      {
      if (rc==SQL_NO_DATA_FOUND)
        break;      
      else checkrc(RTN_OK, henv, hdbc, hstmt); 
      }
    }
EXIT:    
  SQLFreeStmt(hstmt, SQL_DROP);
  checkrc(RTN_OK, henv, hdbc, hstmt);
}

/****************************************************************************
 Show some HTML code for reQuery 
 ****************************************************************************/
static void showRequery(FNAME TmpFn)
{
  printf("\n<script language=\"JavaScript\"> \n");
  printf("function check_input() { \n");
  printf("  var status=false; \n");
  printf("   if (document.query.addPattern.value != \"\") \n");
  printf("      status=true; \n");
  printf("   if (status == false) \n");
  printf("      alert(\"%s\"); \n", PIPATTERN);
  printf("  return status; \n");
  printf("}; \n");
  printf("</script> \n");
  
  printf("<form name=query METHOD=\"POST\" ACTION=\"search.cgi\" onsubmit=\"return check_input()\"> \n");
  printf(" <input type=hidden name=TmpFn value=\"%s\"> \n", TmpFn);
  printf("  <input type=text name=addPattern value=\"\" SIZE=40 > \n");
  printf("  <input TYPE=submit name=requery value=\"%s\"><br> \n", REQUERY);
  printf("  Case sensitive? \n");
  printf("  <input type=radio name=\"c-sensitive\" value=1>Yes \n");
  printf("  &nbsp;&nbsp;&nbsp; \n");
  printf("  <input type=radio name=\"c-sensitive\" value=0 checked>No \n");
  printf("</form> \n");
}

/****************************************************************************
 Show the footer content after SEARCHTAG in template html file.
 (The search result is shown above the template footer.)
 ****************************************************************************/
static void showTemplateFooter(
  HENV henv, 
  HDBC hdbc)
{
  HSTMT   hstmt;
  SDWORD  cbfooter;
  SDWORD  rc;
  char    footer[1024];
  char    buf[1000];
  
  SQLAllocStmt(hdbc,&hstmt);
  checkrc(RTN_OK, henv, hdbc, hstmt);  
  
  sprintf(buf, "select footer from SYSADM.Output");
  SQLExecDirect(hstmt, (UCHAR *)buf, SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, hstmt);
  while (TRUE)
    {
    rc = SQLFetch(hstmt);
    if (rc==SQL_SUCCESS || rc==SQL_SUCCESS_WITH_INFO)
      {
      while (TRUE)
        {  
        rc = SQLGetData(hstmt, 1, SQL_C_CHAR, footer, 1024, &cbfooter);
        if (rc==SQL_SUCCESS)
          {
          printf("%s",footer);
          break;
          }
        else if( rc==SQL_SUCCESS_WITH_INFO)
          {
          printf("%s",footer);
          }
        else checkrc(RTN_OK, henv, hdbc, hstmt); 
        }  
      }
    else 
      {
      if (rc==SQL_NO_DATA_FOUND)
        break;      
      else checkrc(RTN_OK, henv, hdbc, hstmt); 
      }
    }

EXIT:    
  SQLFreeStmt(hstmt, SQL_DROP);
  checkrc(RTN_OK, henv, hdbc, hstmt);
}

/****************************************************************************
 Used for reQuery operation, combine the original search pattern 
 with the new query pattern
 ****************************************************************************/
static int getFullPattern(
  char *pattern,        /* IN added pattern word */
  char *fuPattern,      /* OUT full pattern word */
  FNAME TmpFn)          /* IN tempoary file name */
{
  FILE   *tf=NULL;
  HEADER header;
  char   lptn[LEN_PAT];
  char   rptn[LEN_PAT];

  if ((tf=fopen(TmpFn, "rb"))== NULL)
    {
    printf("File open error %d of getFullPattern\n", errno); 
    return -1;
    }
  
  fseek(tf, 0L, SEEK_SET);
  fread(&header, sizeof(header), 1, tf);

  if (header.pattern[0])
    {
    if (strchr(header.pattern, '|'))
      sprintf(lptn, "(%s)", header.pattern);
    else 
      strcpy(lptn, header.pattern);
      
    if (strchr(pattern, '|'))
      sprintf(rptn, "(%s)", pattern);
    else 
      strcpy(rptn, pattern);
    
    if (strlen(lptn)+strlen(rptn) +1 >= LEN_FUPAT )
      {
      printf("Error: Query pattern too long, please change pattern \n");
      return -1;
      }
    sprintf(fuPattern, "%s&%s", lptn, rptn);
    } 
  else 
    {
    strcpy(fuPattern, pattern);  
    }

  if (tf) 
    fclose(tf);  
  return 0;
}

/****************************************************************************
 Just show some HTML code 
 ****************************************************************************/
static void showForm(
  int pageNumber,       /* IN page number of search page */
  int hitPerP,          /* IN hit count per page */
  int total,            /* IN total count */
  FNAME TmpFn)          /* IN temporary file name */
{
  printf("\n<script language=\"javascript\"> \n");
  printf("    function changePagenumber(tar, modify, tc) \n");
  printf("    {          \n         ");
  printf("      var ans; \n");
  printf("      ans=eval(parseInt(tar, 10) + parseInt(modify, 10)); \n");
  printf("      if (ans > parseInt(tc, 10)) \n ");
  printf("        { ans=eval(parseInt(tar, 10)); } \n ");
  printf("      if (ans < 0 ) \n ");
  printf("        { ans=0; } \n");
  printf("      return ans; \n");
  printf("    }    \n");
  printf("</script>\n");

  printf("<form name=cont method=\"POST\" action=\"search.cgi\">\n");
  printf("<input type=hidden name=pageNumber value=\"%d\">\n",pageNumber);
  printf("  <input type=hidden name=hitPerP value=\"%d\">\n",hitPerP);
  printf("  <input type=hidden name=total value=\"%d\">\n",total);  
  printf("  <input type=hidden name=TmpFn value=\"%s\">\n",TmpFn);
      
  if (pageNumber > 0)
    {
    printf("<input type=submit name=First value=\"%8s\" "
           "  onClick=\"document.cont.pageNumber.value='0'\"> \n", ONE);      
    printf("<input type=submit name=Prev value=\"%8s\" "
           "  onClick=\"document.cont.pageNumber.value="
           "     changePagenumber(document.cont.pageNumber.value,-1, %d)\"> \n", 
           PREV, total/hitPerP);
    }
  if (pageNumber < (total-1)/hitPerP)
    {
    printf("<input type=submit name=Next value=\"%8s\" "
           "  onClick=\"document.cont.pageNumber.value="
           "    changePagenumber(document.cont.pageNumber.value,1, %d)\">\n", 
           NEXT, total/hitPerP);
    printf("<input type=submit name=Last value=\"%8s\" "
           "  onClick=\"document.cont.pageNumber.value='%d'\"> \n", 
           LAST, (total-1)/hitPerP);
    }
  printf("</form> \n");
}

/****************************************************************************
 The function is used to process page-down and page-up 
 ****************************************************************************/
static void proContinue(
  int pageNumber,   /* IN page number */
  int hitPerP,      /* IN hit count per page */
  int total,        /* IN total count */
  FNAME TmpFn)      /* IN temporary file name */
{
  HENV    henv;
  HDBC    hdbc;
  HSTMT   hstmt;
  
  if (SQLAllocEnv(&henv))
    printf("allocate envrionment handle fail !! <br>\n");
  if (SQLAllocConnect(henv,&hdbc))
    printf("allocate connection handle fail !! <br>\n");

  /*=== User Guest with priviledge DBA Connect to Database DBNAME ===*/
  SQLConnect(hdbc, (UCHAR *)DBNAME, SQL_NTS, (UCHAR *)"guest", SQL_NTS,
             (UCHAR *)"", SQL_NTS);
  checkrc(RTN_OK, henv, hdbc, NULL);
  SQLSetConnectOption(hdbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_OFF);
  checkrc(RTN_OK, henv, hdbc, NULL);
  SQLAllocStmt(hdbc,&hstmt);
  checkrc(RTN_OK, henv, hdbc, hstmt);  
  
  mapClientText(henv, hdbc); 
  showTemplateHeader(henv, hdbc);
  showRequery(TmpFn);
  readItem(henv, hdbc, TmpFn, pageNumber, hitPerP);
  showForm(pageNumber, hitPerP, total, TmpFn);
  showTemplateFooter(henv, hdbc); 
  printf("</body></html>");

EXIT:  
  SQLDisconnect(hdbc);
  SQLFreeConnect(hdbc);
  SQLFreeEnv(henv);
  return;
}

