/****************************************************************************
*    Copyright (c) 1999 CASEMaker Inc.                                      *
*    http://www.dbmaker.com  http://www.dbmaker.com.tw                      *
*    You may distribute under the terms of the GNU General Public License.  *
*                                                                           *
*    webut.c                                                                *
*    Functions which a web administrator and web client will use in common. *
*****************************************************************************/
#include "webut.h"

/****************************************************************************
 Just copy content of filename to fcontent
 ****************************************************************************/
void copyFile(
  char *filename,  /* IN file name */
  char *fcontent,  /* OUT file content */
  int  fileLen)    /* IN file content size */
{
  FILE *fp=NULL;
  int  realSize;

  if ((fp=fopen(filename,"r")) == NULL) 
    { printf("html file open error\t%s<br>\n", filename); }
  else
    {
    if ((realSize=fread(fcontent,sizeof(char),fileLen,fp)) == 0)
      { printf("file %s read error<br>\n", filename); }
    }
    fcontent[realSize]='\0';

  if (fp) 
    fclose(fp);
}

/***********************************************************************.
 *   _checkrc  --  check rc and show related info 
.***********************************************************************/
SDWORD _checkrc(
  SDWORD rightrc,   /* IN right rc */
  HENV   xhenv,     /* IN environment handle */
  HDBC   xhdbc,     /* IN data source handle */
  HSTMT  xstmt,     /* IN statement handle */
  char   *filename, /* IN source file */
  int    lineno)    /* IN source line number */
{
  UCHAR   outmsg[200],xstate[10];
  SWORD   size=100;
  SWORD   msglen;
  SDWORD  rc=0;
  SQLError(xhenv,xhdbc,xstmt,xstate,&rc,outmsg,size,&msglen);
  if (rc != rightrc)
    {
    printf("<br>------------------ <font color=\"red\">Error</font> ------------------\n"
		   "<br>file=%s  , Line %d , Unexpected rc: %ld\n"
           "<br>Error message: %s\n"
           "<br>-------------------------------------------\n", 
           filename, lineno, rc, outmsg); 
    return(-1);
    }
  return(0);
}

/****************************************************************************
 strCaseStrOffset is a function similiar with strstr of C library function, 
 it will check whether the source contain pattern, 
 then return the find address of target   
 ****************************************************************************/
int strCaseStrOffset(
  char *source,     /* IN source string */
  char *pattern)    /* IN target pattern string */
{
  int aux;
  int lenOfSource;
  int lenOfPattern;;

  lenOfSource=strlen(source);
  lenOfPattern=strlen(pattern);
  
  for (aux=0; aux< lenOfSource; aux++)
    {
    if (toupper(source[aux]) == toupper(pattern[0]))
      {
#ifndef WIN32
      if (strncasecmp(source+aux, pattern, lenOfPattern)==0)
#else
      if (strnicmp(source+aux, pattern, lenOfPattern)==0)
#endif      
        return aux;
      }
    } 
  return aux;
}/* end of strCaseStrOffset */

/***********************************************************************
  Find the assigned tag 
  OUTPUT:
     1: found tag
     0: not found
 ************************************************************************/
int findTag(
  char *html,   /* IN source html content */
  int slen,     /* IN source html content size */
  char *tag)    /* IN tag  */
{
  int s, tagLen=strlen(tag);

  for (s=0; s<slen; s++)  
    {
#ifndef WIN32
    if ((html[s] =='<') && (strncasecmp(html+s, tag, tagLen) ==0))
#else
    if ((html[s] =='<') && (strnicmp(html+s, tag, tagLen) ==0))
#endif
      return 1;
    }
  return 0;
}

/****************************************************************************
 Check whether the file contain the assigned tag.
 If the file contain the assigned tag, return 0, else return 1.
 ****************************************************************************/
int check(
  char *fname,   /* IN source file name */
  char *tag)     /* IN target tag string */
{
  char *ftmp=NULL;
  struct stat statbuf;
  int fsize, right=0;

  if (stat(fname, &statbuf) != 0 )
      {
      printf("\nFile %s get stat information error, no such file or directory", fname);
      return (1);
      }
  else
    {
    fsize=statbuf.st_size;
    if ( (ftmp=(char *)malloc(fsize+1))== NULL )
      {
      printf("Memory Allocate error");
      goto EXIT;
      }
    copyFile(fname, ftmp, fsize);
    right=findTag(ftmp, fsize, tag);
    }
EXIT:
  if (ftmp) 
    free(ftmp);
  if (right)
    printf("<br>The template html file contain the necessary tag<br>\n");
  else 
    {
    printf("<br>The template html file doesn't contain the necessary tag,");
    printf("please go the the last page to reassign the template file<br>\n");
    return 1;
    }
  return 0;
}

#ifdef WIN32
/* int PUBLIC gettimeofday (DW *val,DW *zone) */
int gettimeofday (
  struct timeval *val, 
  DW *zone)
{
  time_t tc;
  struct _timeb tb;
 
  time( &tc ) ;
  _ftime( &tb ) ;
  /* val->i1 = tc; */  
  val->tv_sec = tc; 

  /* val->i2 = tb.millitm * 1000L ; */    
  val->tv_usec = tb.millitm * 1000L;
  if (zone != NULL)  
  {
    zone->i1 = tb.timezone;
    zone->i2 = tb.dstflag;
  }
  return( 0 ) ;
}
#else
#endif /* DB_PCWIN */
