/****************************************************************************
*    Copyright (c) 1999 CASEMaker Inc.                                      *
*    http://www.dbmaker.com  http://www.dbmaker.com.tw                      *
*    You may distribute under the terms of the GNU General Public License.  *
*                                                                           *
*    webut.h                                                                *
*    Function prototype which are used for web administrator and web client *
*    in common.                                                             *
*****************************************************************************/

#ifndef WIN32
#include <dirent.h>
#include <sys/time.h>
#include <unistd.h>
#include "sqlunix.h"
#else
#include <io.h>
#include <windows.h>
#include <sys/timeb.h>
#endif

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>  
#include <ctype.h>

#include "sqlext.h"
#include "sqlopt.h"

/* is English alphabet or number digit  */
#define isAlNum(ch) (isalnum((unsigned char)(ch)))

/* convert a char to upper case   */
#define toUpper(ch) (toupper((unsigned char)(ch)))

/*************************************************************************
 *  macros for Double Byte Character checking
 *  big5 range, 
 *         high byte : 0x81-0xFF (81-9f:new, a1-a3:symbol, a4-fe:chinese
 *         low byte  : 0x40-0x7E, 0xA1-0xFE
 *  gbcode range,
 *         high byte : 0xA1-0xFE
 *         low byte  : 0xA1-0xFE
 *  Shift JIS,
 *         high byte : 0x81-0x9F, 0xE0-0xFC
 *         low byte  : 0x40-0x7E, 0x80-0xFC
 *  Half Corner(Japanese Single byte character, compatible with Shift JIS)
 *         byte : 0xA1 - 0xDF
 *************************************************************************/
/* is high byte of double byte character     */
#define isDbcsHi(hi) ((unsigned char)(hi) >= 0x81)


#ifndef DBNAME
#define DBNAME "WEBSEARCH"
#endif

#define LEN_FNAME   256       /* define length of file name */
#define LEN_MES     50        /* define length of message */
#define LEN_PAT     200       /* define length of pattern word */
#define LEN_FUPAT   400       /* define length of full pattern word */
#define LEN_BUF     4096      /* define length of buffer */
#define LEN_TAG     30        /* define length of tag */
#define LEN_TITLE   1000      /* define length of HTML title */
#define LEN_PATHS   1000      /* define length of PATH */
#define LEN_PWD     20        /* define length of password */
#define LEN_EXT     100       /* define length of assigned filename extensions */
#define LEN_PARAM   10        /* define length of chValue of parameter */
#define LEN_NAME    10        /* define length of name */
#define LEN_CHVALUE 30        /* define length of chValue */
#define LEN_HTML    10        /* define length of HTML word */
#define LEN_TIME    26        /* define length of time record */
#define LEN_FRONT  280        /* define length in front of searched pattern word in description */
#define LEN_BACK   400        /* define length in back of searched pattern in description */

#define INI_HIP     10        /* initialized value of hit count per page */
#define INI_WGT     10        /* initialized value of weight of title */
#define INI_REC      0        /* whether recursively process sub-directory, 1 means yes and 0 means not*/
#define INI_LOG      0        /* whether log operation of client, 1 means yes and 0 means not */

#define COUNT_EXT   20        /* count of filename extensions */
#define COUNT_DIR   20        /* count of directories */

#define TIME_EXPIRE 1800      /* time expiration to clear temporary file in temp directory */
#define TRUE        1
#define FALSE       0
#define RTN_OK      0
#define SEARCHTAG   "<!--$DBMSearch$-->"
#define MAXROW      200       /* the maximal row count we consider the search result is too many */


typedef char FNAME[LEN_FNAME];  /* type of file name */

typedef struct                /* type of double word timestamp */
{
	unsigned long i1;
	unsigned long i2;
} DW;

/* provide three data types for web search */
typedef struct tOid {
  unsigned char data[8];
} OID;

typedef struct tHeader {
  int  total;                 /* total searched result count */
  char pattern[LEN_FUPAT];    /* search pattern word */
  char CGIpattern[LEN_FUPAT]; /* modified search pattern(conver '&' to '%26') */
  int  hitPerP;               /* hit count item shown per page */
  int  sensitive;             /* whether case-sensitive */
} HEADER;

typedef struct tItem {
  int  num;                   /* serial number of the item in the searched result */ 
  char title[LEN_TITLE];      /* title of the html */
  char fname[LEN_FNAME];      /* filename of the html file */
  char oid[16];
  int  score;                 /* score of the searched item */
} ITEM;
/*  end the three types for web search */

#ifndef WIN32
#define SLASH '/'

#define WS_OPENDIR(dirName, ptr, handle, rc)\
{\
  rc = 0;\
  handle = opendir(dirName);\
  if (handle == NULL)\
    rc = 1;\
}

#define WS_READDIR(handle, ptr, rc)\
{\
  rc = 0;\
  ptr=readdir(handle);\
  if (ptr==NULL)\
    rc = 1;\
}

#define WS_CLOSEDIR(handle)\
{\
  closedir(handle);\
}

#define WS_GETFILENAME(direntp, filename)\
{\
  filename=direntp->d_name;\
}

#define WS_DELETEFILE(filename, command)\
{\
  sprintf(command, "rm -f %s", filename);\
  system(command);\
}

#else  /* WIN32 */

#define SLASH '\\'

#define WS_OPENDIR(dirName, ptr, handle, rc)\
{\
  rc = 0;\
  handle = FindFirstFile(dirName, ptr);\
  if (handle == INVALID_HANDLE_VALUE)\
    rc = 1;\
} 

#define WS_READDIR(handle, ptr, rc)\
{\
  if (FindNextFile(handle, ptr))\
    rc=0;\
  else\
    rc = 1;  \
}

#define WS_CLOSEDIR(handle)\
{\
  FindClose(handle);\
}

#define WS_GETFILENAME(direntp, filename)\
{\
  filename=direntp->cFileName;\
}

#define WS_DELETEFILE(filename, command)\
{\
  remove(filename);\
}

#endif

#define checkrc(rightrc, xhenv, xhdbc, xcmdp)\
  { if (_checkrc(rightrc,xhenv,xhdbc,xcmdp,__FILE__, __LINE__))\
      goto EXIT;\
  }
  
void copyFile(char *filename, char *fcontent, int fileLen);
SDWORD _checkrc(SDWORD rightrc, HENV xhenv, HDBC xhdbc, HSTMT xcmdp, char *filename, int lineno);

int strCaseStrOffset(char *content, char *searchString);
/*=== Use strCaseStrOffset function to find out where the offset 
      from beinning of content to searchString postion case-insensitively ===*/

int findTag(char *html, int slen, char *tag);
int check(char *fname, char *tag);

#ifdef WIN32
int gettimeofday (struct timeval *val, DW *zone);
#else
#endif
